@@ -1,7 +1,12 @@
 
 // Start Farabi when the document loads
 $(function() {
-	var editorId = "editor";
+	
+	// Editors hash object
+	var editors = {};
+
+	// Current Editor
+	var currentEditorId;
 	
 	// Cache dialog jQuery references for later use
 	var $about_dialog = $('#about-dialog');
@@ -23,11 +28,8 @@ $(function() {
 		}
 	}
 
-	// Editors hash object
-	var editors = {};
-
 	var tabId = 1;
-	window.AddEditorTab = function(tabName) {
+	window.AddEditorTab = function(tabName, filename, mode, contents) {
 		
 		if(!tabName) {
 			// Tab name
@@ -35,10 +37,10 @@ $(function() {
 		}
 		
 		// Generate editor id
-		editorId = "editor" + tabId;
+		var editorId = "editor" + tabId;
 
 		// Add the tab component
-		$("#myTab").append('<li><a href="#' + tabId + '" data-toggle="tab">' + tabName + '</a></li>');
+		$("#myTab").append('<li><a href="#' + tabId + '" data-toggle="tab">' + tabName + '&nbsp;<span></span>&nbsp;&nbsp;<i class="icon-remove"></i></a></li>');
 
 		// Add the tab editor
 		$("#myTabContent").append(
@@ -54,7 +56,33 @@ $(function() {
 					'</div>' +
 				'</div>' +
 			'</div>');
+			
+		$("i", "#myTab").mouseover(function() {
+			$(this).addClass("farabi-icon-selected");
+		});
 		
+		$("i", "#myTab").mouseout(function() {
+			$(this).removeClass("farabi-icon-selected");
+		});
+		
+		// When the close file icon is clicked
+		$("i", "#myTab").click(function(e) {
+			
+			// Update currentEditorId
+			var $tab = $(this).parent();
+			var href = $tab.attr("href");
+			currentEditorId = "editor" + href.substring(1);
+			
+			currentEditor = editors[currentEditorId].editor;
+
+			// Stop event propagation right away
+			e.stopImmediatePropagation();
+
+			// Trigger close file action
+			$(document).trigger('action-close-file');
+
+		});
+
 		// Create the CodeMirror editor instance
 		var editor = CodeMirror.fromTextArea(document.getElementById(editorId), {
 			//TODO these should be configurable!
@@ -72,6 +100,21 @@ $(function() {
 				}
 			}
 		});
+		
+		// Set the contents if available
+		if(contents) {
+			editor.setValue(contents);
+			editor.clearHistory();
+			editor.markClean();
+		}
+
+		// Change the mode...
+		if( !mode ) {
+			// Default to Perl language
+			window.ChangeMode(editor, 'perl');
+		} else {
+			window.ChangeMode(editor, mode);
+		}
 
 		// Hook up with cursor activity event
 		editor.on("cursorActivity", function(cm) {
@@ -86,7 +129,7 @@ $(function() {
 				showEditorStats(cm);
 		});
 
-		// Hook up with cursor activity event
+		// Hook up with change event
 		var timeoutId;
 		editor.on("change", function(cm) {
 				clearInterval(timeoutId);
@@ -96,17 +139,32 @@ $(function() {
 						podChanged(cm);
 						podCheck();
 					} else {
-						$("#pod-tab").html('Not supported for mode "' + mode + '"');
+						$("#pod-tab").html('Only supported for Perl files');
 					}
+
 				}, 250);
+
+				// Show a '*' after the tab name if it is modified
+				var isClean = cm.isClean();
+				if(isClean) {
+					$("#myTab > li.active > a > span").text('');
+				} else {
+					$("#myTab > li.active > a > span").text(" *");
+				}
+				
 		});
 
 		
 		// Store editor reference
-		editors[editorId] = editor;
+		editors[editorId] = {
+			editor   : editor,
+			filename : filename,
+			tabName  : tabName,
+			tabId    : tabId
+		};
 		
-		// This is the current editor
-		currentEditor = editor;
+		// This is the current editor identifier
+		currentEditorId = editorId;
 
 		// Show the last editor tab which we added
 		var $lastTab = $("#myTab a:last");
@@ -119,9 +177,6 @@ $(function() {
 
 		// Run these when we exit this one
 		setTimeout(function() {
-			// Editor is by default Perl
-			ChangeMode(editor, 'perl');
-
 			// focus!
 			editor.focus();
 
@@ -151,9 +206,11 @@ $(function() {
 	};
 	
 
-	// Add an empty one editor tab on startup
-	AddEditorTab();
-
+	// Disable all form submission to prevent UI from refreshing by mistake on ENTER
+	$("form").submit(function() {
+		return false;
+	});
+	
 	function _displayHelp(topic, bShowDialog) {
 		$.post('/help_search', {"topic": topic}, function(results) {
 			if(results.length > 0) {
@@ -189,17 +246,18 @@ $(function() {
 		CodeMirror.autoLoadMode(cm, modeFile);	
 	}
 
-	var plural = function(number) {
-		if(number == 1) {
-			return 'st';
-		} else if(number == 2) {
-			return 'nd';
-		} else {
-			return 'th';
-		}
-	}
-
 	var showEditorStats = function(cm) {
+		
+		var plural = function(number) {
+			if(number == 1) {
+				return 'st';
+			} else if(number == 2) {
+				return 'nd';
+			} else {
+				return 'th';
+			}
+		};
+		
 		var cursor = cm.getCursor();
 		var selection = cm.getSelection();
 		var line_number = cursor.line + 1;
@@ -220,12 +278,16 @@ $(function() {
 		});
 	};
 
-	/** Get the current editor */
+	// Get the current editor
 	window.GetCurrentEditor = function() {
-		//TODO return a better one :)
-		return currentEditor;
+		return editors[currentEditorId].editor;
 	};
 	
+	// Returns the current editor filename
+	window.GetCurrentFilename = function() {
+		return editors[currentEditorId].filename;
+	}
+	
 	$(".results").hide();
 
 	$(document).on('action-perl-doc', function() {
@@ -316,7 +378,10 @@ $(function() {
 	});
 
 	var onCloseFocusEditor = function () {
-		GetCurrentEditor().focus();
+		var editor = GetCurrentEditor();
+		if(!editor) {
+			editor.focus();
+		}
 	};
 
 	// All modal dialogs are now hidden on startup
@@ -345,4 +410,67 @@ $(function() {
 		AddEditorTab();
 	});
 
+	
+	$(document).on('action-close-file', function() {
+		
+		var o = editors[currentEditorId];
+		if(!o.editor.isClean()) {
+			// Can it be safely close file without losing data?
+			$("#save-question-dialog").modal('show');
+			return;
+		}
+
+		// Utility function to find if the object is empty or not
+		var isEmpty = function (obj) {
+		    for (var key in obj) {
+			    return false;
+		    }
+		    return true;
+		}
+		
+		// Remove the current tab from the UI
+		var currentTabId = editors[currentEditorId].tabId;
+		$("#myTab > li > a[href=#" + currentTabId + "]").remove();
+		$("#myTabContent > div[id=" + currentTabId + "]").remove();
+
+		// Remove it from editors object
+		delete editors[currentEditorId];
+		
+		if(isEmpty(editors)) {
+			// Add a default tab if it was the last tab
+			AddEditorTab();
+		} else {
+			// Show the last editor tab which we added
+			$("#myTab a:last").tab('show');
+		}
+	});
+
+	$(document).on('action-close-all-files', function() {
+		$("#myTab > li").remove();
+		$("#myTabContent > div").remove();
+
+		// Empty all editors
+		editors = {};
+
+		// Add a default tab since there is none
+		AddEditorTab();
+	});
+
+	$(document).on('action-save-file', function() {
+		var editor = GetCurrentEditor();
+		var filename = GetCurrentFilename();
+		$.post('/save-file', {"filename": filename, "source": editor.getValue()}, function(data) {
+			if(data.error == '') {
+				alert("File saved!");
+			} else {
+				alert('Error:\n' + data.error);
+			}
+		});
+	});
+
+	
+	
+	// Add an empty one editor tab on startup
+	AddEditorTab();
+
 });
\ No newline at end of file
